from typing import Any, Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import BaseSDK, Timer
from anyscale.cli_logger import BlockLogger
from anyscale.llm.model._private.models_sdk import PrivateLLMModelsSDK
from anyscale.llm.model.commands import (
    _DELETE_ARG_DOCSTRINGS,
    _DELETE_EXAMPLE,
    _LIST_ARG_DOCSTRINGS,
    _LIST_EXAMPLE,
    _RETRIEVE_ARG_DOCSTRINGS,
    _RETRIEVE_EXAMPLE,
)


class LLMModelsSDK(BaseSDK):
    def __init__(
        self,
        *,
        logger: Optional[BlockLogger] = None,
        client: Optional[AnyscaleClientInterface] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateLLMModelsSDK(
            logger=logger, client=client, timer=timer
        )

    @sdk_docs(
        doc_py_example=_RETRIEVE_EXAMPLE, arg_docstrings=_RETRIEVE_ARG_DOCSTRINGS,
    )
    def get(
        self, *, model_id: Optional[str] = None, job_id: Optional[str] = None
    ) -> Any:
        """Retrives model card for a finetuned model."""
        return self._private_sdk.get(model_id=model_id, job_id=job_id)

    @sdk_docs(
        doc_py_example=_DELETE_EXAMPLE, arg_docstrings=_DELETE_ARG_DOCSTRINGS,
    )
    def delete(self, model_id: str) -> Any:
        """Deletes a finetuned model. Requires owner permission for the corresponding Anyscale project."""
        return self._private_sdk.delete(model_id)

    @sdk_docs(
        doc_py_example=_LIST_EXAMPLE, arg_docstrings=_LIST_ARG_DOCSTRINGS,
    )
    def list(
        self,
        *,
        cloud_id: Optional[str] = None,
        project_id: Optional[str] = None,
        max_items: int = 20,
    ) -> Any:
        """Lists fine-tuned models available to the user.

        By default, all models in all visible clouds under all visible projects to the user are listed. This is optionally filtered by `project_id` and/or `cloud_id`.
        """
        return self._private_sdk.list(
            cloud_id=cloud_id, project_id=project_id, max_items=max_items
        )



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class FineTunedModel(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'base_model_id': 'str',
        'creator_id': 'str',
        'creator': 'MiniUser',
        'created_at': 'datetime',
        'storage_uri': 'str',
        'generation_config': 'object',
        'ft_type': 'FineTuneType',
        'cloud_id': 'str',
        'project_id': 'str',
        'job_id': 'str',
        'workspace_id': 'str'
    }

    attribute_map = {
        'id': 'id',
        'base_model_id': 'base_model_id',
        'creator_id': 'creator_id',
        'creator': 'creator',
        'created_at': 'created_at',
        'storage_uri': 'storage_uri',
        'generation_config': 'generation_config',
        'ft_type': 'ft_type',
        'cloud_id': 'cloud_id',
        'project_id': 'project_id',
        'job_id': 'job_id',
        'workspace_id': 'workspace_id'
    }

    def __init__(self, id=None, base_model_id=None, creator_id=None, creator=None, created_at=None, storage_uri=None, generation_config=None, ft_type=None, cloud_id=None, project_id=None, job_id=None, workspace_id=None, local_vars_configuration=None):  # noqa: E501
        """FineTunedModel - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._base_model_id = None
        self._creator_id = None
        self._creator = None
        self._created_at = None
        self._storage_uri = None
        self._generation_config = None
        self._ft_type = None
        self._cloud_id = None
        self._project_id = None
        self._job_id = None
        self._workspace_id = None
        self.discriminator = None

        self.id = id
        self.base_model_id = base_model_id
        if creator_id is not None:
            self.creator_id = creator_id
        if creator is not None:
            self.creator = creator
        self.created_at = created_at
        self.storage_uri = storage_uri
        if generation_config is not None:
            self.generation_config = generation_config
        self.ft_type = ft_type
        self.cloud_id = cloud_id
        if project_id is not None:
            self.project_id = project_id
        if job_id is not None:
            self.job_id = job_id
        if workspace_id is not None:
            self.workspace_id = workspace_id

    @property
    def id(self):
        """Gets the id of this FineTunedModel.  # noqa: E501


        :return: The id of this FineTunedModel.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this FineTunedModel.


        :param id: The id of this FineTunedModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def base_model_id(self):
        """Gets the base_model_id of this FineTunedModel.  # noqa: E501


        :return: The base_model_id of this FineTunedModel.  # noqa: E501
        :rtype: str
        """
        return self._base_model_id

    @base_model_id.setter
    def base_model_id(self, base_model_id):
        """Sets the base_model_id of this FineTunedModel.


        :param base_model_id: The base_model_id of this FineTunedModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and base_model_id is None:  # noqa: E501
            raise ValueError("Invalid value for `base_model_id`, must not be `None`")  # noqa: E501

        self._base_model_id = base_model_id

    @property
    def creator_id(self):
        """Gets the creator_id of this FineTunedModel.  # noqa: E501


        :return: The creator_id of this FineTunedModel.  # noqa: E501
        :rtype: str
        """
        return self._creator_id

    @creator_id.setter
    def creator_id(self, creator_id):
        """Sets the creator_id of this FineTunedModel.


        :param creator_id: The creator_id of this FineTunedModel.  # noqa: E501
        :type: str
        """

        self._creator_id = creator_id

    @property
    def creator(self):
        """Gets the creator of this FineTunedModel.  # noqa: E501


        :return: The creator of this FineTunedModel.  # noqa: E501
        :rtype: MiniUser
        """
        return self._creator

    @creator.setter
    def creator(self, creator):
        """Sets the creator of this FineTunedModel.


        :param creator: The creator of this FineTunedModel.  # noqa: E501
        :type: MiniUser
        """

        self._creator = creator

    @property
    def created_at(self):
        """Gets the created_at of this FineTunedModel.  # noqa: E501


        :return: The created_at of this FineTunedModel.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this FineTunedModel.


        :param created_at: The created_at of this FineTunedModel.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def storage_uri(self):
        """Gets the storage_uri of this FineTunedModel.  # noqa: E501


        :return: The storage_uri of this FineTunedModel.  # noqa: E501
        :rtype: str
        """
        return self._storage_uri

    @storage_uri.setter
    def storage_uri(self, storage_uri):
        """Sets the storage_uri of this FineTunedModel.


        :param storage_uri: The storage_uri of this FineTunedModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and storage_uri is None:  # noqa: E501
            raise ValueError("Invalid value for `storage_uri`, must not be `None`")  # noqa: E501

        self._storage_uri = storage_uri

    @property
    def generation_config(self):
        """Gets the generation_config of this FineTunedModel.  # noqa: E501


        :return: The generation_config of this FineTunedModel.  # noqa: E501
        :rtype: object
        """
        return self._generation_config

    @generation_config.setter
    def generation_config(self, generation_config):
        """Sets the generation_config of this FineTunedModel.


        :param generation_config: The generation_config of this FineTunedModel.  # noqa: E501
        :type: object
        """

        self._generation_config = generation_config

    @property
    def ft_type(self):
        """Gets the ft_type of this FineTunedModel.  # noqa: E501


        :return: The ft_type of this FineTunedModel.  # noqa: E501
        :rtype: FineTuneType
        """
        return self._ft_type

    @ft_type.setter
    def ft_type(self, ft_type):
        """Sets the ft_type of this FineTunedModel.


        :param ft_type: The ft_type of this FineTunedModel.  # noqa: E501
        :type: FineTuneType
        """
        if self.local_vars_configuration.client_side_validation and ft_type is None:  # noqa: E501
            raise ValueError("Invalid value for `ft_type`, must not be `None`")  # noqa: E501

        self._ft_type = ft_type

    @property
    def cloud_id(self):
        """Gets the cloud_id of this FineTunedModel.  # noqa: E501


        :return: The cloud_id of this FineTunedModel.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this FineTunedModel.


        :param cloud_id: The cloud_id of this FineTunedModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    @property
    def project_id(self):
        """Gets the project_id of this FineTunedModel.  # noqa: E501


        :return: The project_id of this FineTunedModel.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this FineTunedModel.


        :param project_id: The project_id of this FineTunedModel.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def job_id(self):
        """Gets the job_id of this FineTunedModel.  # noqa: E501


        :return: The job_id of this FineTunedModel.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this FineTunedModel.


        :param job_id: The job_id of this FineTunedModel.  # noqa: E501
        :type: str
        """

        self._job_id = job_id

    @property
    def workspace_id(self):
        """Gets the workspace_id of this FineTunedModel.  # noqa: E501


        :return: The workspace_id of this FineTunedModel.  # noqa: E501
        :rtype: str
        """
        return self._workspace_id

    @workspace_id.setter
    def workspace_id(self, workspace_id):
        """Sets the workspace_id of this FineTunedModel.


        :param workspace_id: The workspace_id of this FineTunedModel.  # noqa: E501
        :type: str
        """

        self._workspace_id = workspace_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FineTunedModel):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, FineTunedModel):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class AviaryModelConfigV2(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'base_model_id': 'str',
        'model_id': 'str',
        'generation': 'object',
        'cloud_id': 'str',
        'project_id': 'str',
        'ha_job_id': 'str',
        'workspace_id': 'str',
        'model_metadata': 'object',
        'lora_mirror_config': 'object',
        'full_param_mirror_config': 'object'
    }

    attribute_map = {
        'base_model_id': 'base_model_id',
        'model_id': 'model_id',
        'generation': 'generation',
        'cloud_id': 'cloud_id',
        'project_id': 'project_id',
        'ha_job_id': 'ha_job_id',
        'workspace_id': 'workspace_id',
        'model_metadata': 'model_metadata',
        'lora_mirror_config': 'lora_mirror_config',
        'full_param_mirror_config': 'full_param_mirror_config'
    }

    def __init__(self, base_model_id=None, model_id=None, generation=None, cloud_id=None, project_id=None, ha_job_id=None, workspace_id=None, model_metadata=None, lora_mirror_config=None, full_param_mirror_config=None, local_vars_configuration=None):  # noqa: E501
        """AviaryModelConfigV2 - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._base_model_id = None
        self._model_id = None
        self._generation = None
        self._cloud_id = None
        self._project_id = None
        self._ha_job_id = None
        self._workspace_id = None
        self._model_metadata = None
        self._lora_mirror_config = None
        self._full_param_mirror_config = None
        self.discriminator = None

        self.base_model_id = base_model_id
        self.model_id = model_id
        self.generation = generation
        self.cloud_id = cloud_id
        if project_id is not None:
            self.project_id = project_id
        if ha_job_id is not None:
            self.ha_job_id = ha_job_id
        if workspace_id is not None:
            self.workspace_id = workspace_id
        if model_metadata is not None:
            self.model_metadata = model_metadata
        if lora_mirror_config is not None:
            self.lora_mirror_config = lora_mirror_config
        if full_param_mirror_config is not None:
            self.full_param_mirror_config = full_param_mirror_config

    @property
    def base_model_id(self):
        """Gets the base_model_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The base_model_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._base_model_id

    @base_model_id.setter
    def base_model_id(self, base_model_id):
        """Sets the base_model_id of this AviaryModelConfigV2.


        :param base_model_id: The base_model_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and base_model_id is None:  # noqa: E501
            raise ValueError("Invalid value for `base_model_id`, must not be `None`")  # noqa: E501

        self._base_model_id = base_model_id

    @property
    def model_id(self):
        """Gets the model_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The model_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._model_id

    @model_id.setter
    def model_id(self, model_id):
        """Sets the model_id of this AviaryModelConfigV2.


        :param model_id: The model_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and model_id is None:  # noqa: E501
            raise ValueError("Invalid value for `model_id`, must not be `None`")  # noqa: E501

        self._model_id = model_id

    @property
    def generation(self):
        """Gets the generation of this AviaryModelConfigV2.  # noqa: E501


        :return: The generation of this AviaryModelConfigV2.  # noqa: E501
        :rtype: object
        """
        return self._generation

    @generation.setter
    def generation(self, generation):
        """Sets the generation of this AviaryModelConfigV2.


        :param generation: The generation of this AviaryModelConfigV2.  # noqa: E501
        :type: object
        """
        if self.local_vars_configuration.client_side_validation and generation is None:  # noqa: E501
            raise ValueError("Invalid value for `generation`, must not be `None`")  # noqa: E501

        self._generation = generation

    @property
    def cloud_id(self):
        """Gets the cloud_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The cloud_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this AviaryModelConfigV2.


        :param cloud_id: The cloud_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    @property
    def project_id(self):
        """Gets the project_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The project_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this AviaryModelConfigV2.


        :param project_id: The project_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def ha_job_id(self):
        """Gets the ha_job_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The ha_job_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._ha_job_id

    @ha_job_id.setter
    def ha_job_id(self, ha_job_id):
        """Sets the ha_job_id of this AviaryModelConfigV2.


        :param ha_job_id: The ha_job_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """

        self._ha_job_id = ha_job_id

    @property
    def workspace_id(self):
        """Gets the workspace_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The workspace_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._workspace_id

    @workspace_id.setter
    def workspace_id(self, workspace_id):
        """Sets the workspace_id of this AviaryModelConfigV2.


        :param workspace_id: The workspace_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """

        self._workspace_id = workspace_id

    @property
    def model_metadata(self):
        """Gets the model_metadata of this AviaryModelConfigV2.  # noqa: E501


        :return: The model_metadata of this AviaryModelConfigV2.  # noqa: E501
        :rtype: object
        """
        return self._model_metadata

    @model_metadata.setter
    def model_metadata(self, model_metadata):
        """Sets the model_metadata of this AviaryModelConfigV2.


        :param model_metadata: The model_metadata of this AviaryModelConfigV2.  # noqa: E501
        :type: object
        """

        self._model_metadata = model_metadata

    @property
    def lora_mirror_config(self):
        """Gets the lora_mirror_config of this AviaryModelConfigV2.  # noqa: E501


        :return: The lora_mirror_config of this AviaryModelConfigV2.  # noqa: E501
        :rtype: object
        """
        return self._lora_mirror_config

    @lora_mirror_config.setter
    def lora_mirror_config(self, lora_mirror_config):
        """Sets the lora_mirror_config of this AviaryModelConfigV2.


        :param lora_mirror_config: The lora_mirror_config of this AviaryModelConfigV2.  # noqa: E501
        :type: object
        """

        self._lora_mirror_config = lora_mirror_config

    @property
    def full_param_mirror_config(self):
        """Gets the full_param_mirror_config of this AviaryModelConfigV2.  # noqa: E501


        :return: The full_param_mirror_config of this AviaryModelConfigV2.  # noqa: E501
        :rtype: object
        """
        return self._full_param_mirror_config

    @full_param_mirror_config.setter
    def full_param_mirror_config(self, full_param_mirror_config):
        """Sets the full_param_mirror_config of this AviaryModelConfigV2.


        :param full_param_mirror_config: The full_param_mirror_config of this AviaryModelConfigV2.  # noqa: E501
        :type: object
        """

        self._full_param_mirror_config = full_param_mirror_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AviaryModelConfigV2):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, AviaryModelConfigV2):
            return True

        return self.to_dict() != other.to_dict()



import os

from anyscale.utils.name_utils import get_full_name


def test_get_full_name():
    for env_var in ["ANYSCALE_USERNAME", "USERNAME", "USER"]:
        if os.getenv(env_var):
            os.environ.pop(env_var)

    # Test sourcing priority
    assert get_full_name() == "default"
    os.environ["USER"] = "John Doe"
    assert get_full_name() == "john_doe"
    os.environ["USERNAME"] = "Jane   Smith"
    assert get_full_name() == "jane_smith"
    os.environ["ANYSCALE_USERNAME"] = "Ryan"
    assert get_full_name() == "ryan"



from collections import defaultdict
from dataclasses import dataclass, field
from typing import Any, DefaultDict, Dict, List, Optional, Union

from anyscale._private.models import ModelBase, ModelEnum


ResourceDict = Dict[str, float]
AdvancedInstanceConfigDict = Dict[str, Any]


def _validate_resource_dict(r: Optional[ResourceDict], *, field_name: str):
    if r is None:
        return

    if not isinstance(r, dict):
        raise TypeError(f"'{field_name}' must be a Dict[str, float], but got: {r}")

    for k, v in r.items():
        if not isinstance(k, str):
            raise TypeError(f"'{field_name}' keys must be strings, but got: {k}")
        if isinstance(v, (int, float)):
            if v < 0:
                raise ValueError(
                    f"'{field_name}' values must be >= 0, but got: '{k}: {v}'"
                )
        else:
            raise TypeError(
                f"'{field_name}' values must be floats, but got: '{k}: {v}'"
            )


def _validate_advanced_instance_config_dict(c: Optional[AdvancedInstanceConfigDict]):
    if c is None:
        return

    if not isinstance(c, dict) or not all(isinstance(k, str) for k in c):
        raise TypeError("'advanced_instance_config' must be a Dict[str, Any]")


@dataclass(frozen=True)
class CloudDeployment(ModelBase):
    """Cloud deployment selectors for a node group; one or more selectors may be passed to target a specific deployment from all of a cloud's deployments."""

    __doc_py_example__ = """
from anyscale.compute_config.models import CloudDeployment

cloud_deployment = CloudDeployment(
    provider="aws",
    region="us-west-2",
    machine_pool="machine-pool-name",
    id="cldrsrc_1234567890",
)
"""

    __doc_yaml_example__ = """
cloud_deployment:
  provider: aws
  region: us-west-2
  machine_pool: machine-pool-name
  id: cldrsrc_1234567890
"""

    provider: Optional[str] = field(
        default=None,
        metadata={"docstring": "Cloud provider name, e.g., `aws` or `gcp`."},
    )

    def _validate_provider(self, provider: Optional[str]):
        if provider is not None and not isinstance(provider, str):
            raise TypeError("'provider' must be a string.")

    region: Optional[str] = field(
        default=None,
        metadata={"docstring": "Cloud provider region, e.g., `us-west-2`."},
    )

    def _validate_region(self, region: Optional[str]):
        if region is not None and not isinstance(region, str):
            raise TypeError("'region' must be a string.")

    machine_pool: Optional[str] = field(
        default=None, metadata={"docstring": "Machine pool name."}
    )

    def _validate_machine_pool(self, machine_pool: Optional[str]):
        if machine_pool is not None and not isinstance(machine_pool, str):
            raise TypeError("'machine_pool' must be a string.")

    id: Optional[str] = field(
        default=None, metadata={"docstring": "Cloud deployment ID from cloud setup."}
    )

    def _validate_id(self, id: Optional[str]):  # noqa: A002
        if id is not None and not isinstance(id, str):
            raise TypeError("'id' must be a string.")


@dataclass(frozen=True)
class _NodeConfig(ModelBase):
    instance_type: str = field(
        metadata={
            "docstring": "Cloud provider instance type, e.g., `m5.2xlarge` on AWS or `n2-standard-8` on GCP."
        }
    )

    def _validate_instance_type(self, instance_type: str):
        if not isinstance(instance_type, str):
            raise TypeError("'instance_type' must be a string.")

    resources: Optional[ResourceDict] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Logical resources that will be available on this node. Defaults to match the physical resources of the instance type."
        },
    )

    def _validate_resources(self, resources: Optional[ResourceDict]):
        _validate_resource_dict(resources, field_name="resources")

    advanced_instance_config: Optional[AdvancedInstanceConfigDict] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Advanced instance configurations that will be passed through to the cloud provider.",
            "customer_hosted_only": True,
        },
    )

    def _validate_advanced_instance_config(
        self, advanced_instance_config: Optional[AdvancedInstanceConfigDict]
    ):
        _validate_advanced_instance_config_dict(advanced_instance_config)

    flags: Optional[Dict[str, Any]] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Node-level flags specifying advanced or experimental options.",
            "customer_hosted_only": False,
        },
    )

    def _validate_flags(self, flags: Optional[Dict[str, Any]]):
        if flags is None:
            return

        if not isinstance(flags, dict):
            raise TypeError("'flags' must be a dict")

    cloud_deployment: Union[CloudDeployment, Dict[str, str], None] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Cloud deployment selectors for a node group; one or more selectors may be passed to target a specific deployment from all of a cloud's deployments.",
            "customer_hosted_only": False,
        },
    )

    def _validate_cloud_deployment(
        self, cloud_deployment: Union[CloudDeployment, Dict[str, str], None]
    ) -> Optional[CloudDeployment]:
        if cloud_deployment is None:
            return None
        if isinstance(cloud_deployment, dict):
            cloud_deployment = CloudDeployment.from_dict(cloud_deployment)
        if not isinstance(cloud_deployment, CloudDeployment):
            raise TypeError(
                "'cloud_deployment' must be a CloudDeployment or corresponding dict"
            )
        return cloud_deployment


@dataclass(frozen=True)
class HeadNodeConfig(_NodeConfig):
    """Configuration options for the head node of a cluster."""

    __doc_py_example__ = """
from anyscale.compute_config.models import ComputeConfig, HeadNodeConfig

config = ComputeConfig(
    head_node=HeadNodeConfig(
        instance_type="m5.8xlarge",
    ),
)
"""

    __doc_yaml_example__ = """
head_node:
  instance_type: m5.8xlarge
"""


class MarketType(ModelEnum):
    """Market type of instances to use (on-demand vs. spot)."""

    ON_DEMAND = "ON_DEMAND"
    SPOT = "SPOT"
    PREFER_SPOT = "PREFER_SPOT"

    __docstrings__ = {
        ON_DEMAND: "Use on-demand instances only.",
        SPOT: "Use spot instances only.",
        PREFER_SPOT: (
            "Prefer to use spot instances, but fall back to on-demand if necessary. "
            "If on-demand instances are running and spot instances become available, "
            "the on-demand instances will be evicted and replaced with spot instances."
        ),
    }


@dataclass(frozen=True)
class WorkerNodeGroupConfig(_NodeConfig):
    """Configuration options for a worker node group in a cluster.

    Clusters can have multiple worker node groups that use different instance types or configurations.
    """

    __doc_py_example__ = """
from anyscale.compute_config.models import ComputeConfig, MarketType, WorkerNodeGroupConfig

config = ComputeConfig(
    worker_nodes=[
        WorkerNodeGroupConfig(
            instance_type="m5.8xlarge",
            min_nodes=5,
            max_nodes=5,
        ),
        WorkerNodeGroupConfig(
            instance_type="m5.4xlarge",
            min_nodes=1,
            max_nodes=10,
            market_type=MarketType.SPOT,
        ),
    ],
)
"""

    __doc_yaml_example__ = """
worker_nodes:
- instance_type: m5.8xlarge
  min_nodes: 5
  max_nodes: 5
- instance_type: m5.4xlarge
  min_nodes: 1
  max_nodes: 10
  market_type: SPOT
"""

    name: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "Unique name of this worker group. Defaults to a human-friendly representation of the instance type."
        },
    )

    def _validate_name(self, name: Optional[str]) -> str:
        # Default name to the instance type if not specified.
        if name is None:
            name = self.instance_type

        if not isinstance(name, str):
            raise TypeError("'name' must be a string")
        if len(name) == 0:
            raise ValueError("'name' cannot be empty")

        return name

    min_nodes: int = field(
        default=0,
        metadata={
            "docstring": "Minimum number of nodes of this type that will be kept running in the cluster."
        },
    )

    def _validate_min_nodes(self, min_nodes: int):
        if not isinstance(min_nodes, int):
            raise TypeError("'min_nodes' must be an int")
        if min_nodes < 0:
            raise ValueError("'min_nodes' must be >= 0")

    max_nodes: int = field(
        default=10,
        metadata={
            "docstring": "Maximum number of nodes of this type that can be running in the cluster."
        },
    )

    def _validate_max_nodes(self, max_nodes: int):
        if not isinstance(max_nodes, int):
            raise TypeError("'max_nodes' must be an int")
        if max_nodes < 1:
            raise ValueError("'max_nodes' must be >= 1")
        if max_nodes < self.min_nodes:
            raise ValueError(f"'max_nodes' must be >= 'min_nodes' ({self.min_nodes})")

    market_type: Union[str, MarketType] = field(
        default=MarketType.ON_DEMAND,
        metadata={
            "docstring": "The type of instances to use (see `MarketType` enum values for details).",
            "customer_hosted_only": True,
        },
    )

    def _validate_market_type(self, market_type: Union[str, MarketType]) -> MarketType:
        if isinstance(market_type, str):
            # This will raise a ValueError if the market_type is unrecognized.
            market_type = MarketType(market_type)
        elif not isinstance(market_type, MarketType):
            raise TypeError("'market_type' must be a MarketType.")

        return market_type


@dataclass(frozen=True)
class ComputeConfig(ModelBase):
    """Configuration for instance types and cloud resources for a cluster."""

    __doc_py_example__ = """
from anyscale.compute_config.models import (
    ComputeConfig, HeadNodeConfig, MarketType, WorkerNodeGroupConfig
)

config = ComputeConfig(
    cloud="my-cloud",
    head_node=HeadNodeConfig(
        instance_type="m5.8xlarge",
    ),
    worker_nodes=[
        WorkerNodeGroupConfig(
            instance_type="m5.8xlarge",
            min_nodes=5,
            max_nodes=5,
        ),
        WorkerNodeGroupConfig(
            instance_type="m5.4xlarge",
            min_nodes=1,
            max_nodes=10,
            market_type=MarketType.SPOT,
        ),
    ],
)
"""

    __doc_yaml_example__ = """
cloud: my-cloud
zones: # (Optional) Defaults to to all zones in a region.
  - us-west-2a
  - us-west-2b
head_node:
  instance_type: m5.8xlarge
worker_nodes:
- instance_type: m5.8xlarge
  min_nodes: 5
  max_nodes: 5
  market_type: PREFER_SPOT # (Optional) Defaults to ON_DEMAND
- instance_type: g5.4xlarge
  min_nodes: 1
  max_nodes: 10
  market_type: SPOT # (Optional) Defaults to ON_DEMAND
min_resources: # (Optional) Defaults to no minimum.
  CPU: 1
  GPU: 1
  CUSTOM_RESOURCE: 0
max_resources: # (Optional) Defaults to no maximum.
  CPU: 6
  GPU: 10
  CUSTOM_RESOURCE: 10
enable_cross_zone_scaling: true # (Optional) Defaults to false.
advanced_instance_config: # (Optional) Defaults to no advanced configurations.
  # AWS specific configuration example
  BlockDeviceMappings:
    - DeviceName: DEVICE_NAME
      Ebs:
        VolumeSize: VOLUME_SIZE
        DeleteOnTermination: DELETE_ON_TERMINATION
  IamInstanceProfile:
    Arn: IAM_INSTANCE_PROFILE_ARN
  NetworkInterfaces:
    - SubnetId: SUBNET_ID
      Groups:
        - SECURITY_GROUP_ID
      AssociatePublicIpAddress: ASSOCIATE_PUBLIC_IP
  TagSpecifications:
    - ResourceType: RESOURCE_TYPE
      Tags:
        - Key: TAG_KEY
          Value: TAG_VALUE
  # GCP specific configuration example
  instance_properties:
    disks:
      - boot: BOOT_OPTION
        auto_delete: AUTO_DELETE_OPTION
        initialize_params:
          disk_size_gb: DISK_SIZE_GB
    service_accounts:
      - email: SERVICE_ACCOUNT_EMAIL
        scopes:
          - SCOPE_URL
    network_interfaces:
      - subnetwork: SUBNETWORK_URL
        access_configs:
          - type: ACCESS_CONFIG_TYPE
    labels:
      LABEL_KEY: LABEL_VALUE
"""

    cloud: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace)."
        },
    )

    def _validate_cloud(self, cloud: Optional[str]):
        if cloud is not None and not isinstance(cloud, str):
            raise TypeError("'cloud' must be a string")

    head_node: Union[HeadNodeConfig, Dict, None] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Configuration options for the head node of the cluster. Defaults to the cloud's default head node configuration."
        },
    )

    def _validate_head_node(
        self, head_node: Union[HeadNodeConfig, Dict, None]
    ) -> Optional[HeadNodeConfig]:
        if head_node is None:
            return None

        if isinstance(head_node, dict):
            head_node = HeadNodeConfig.from_dict(head_node)
        if not isinstance(head_node, HeadNodeConfig):
            raise TypeError(
                "'head_node' must be a HeadNodeConfig or corresponding dict"
            )

        return head_node

    worker_nodes: Optional[List[Union[WorkerNodeGroupConfig, Dict]]] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Configuration options for the worker nodes of the cluster. If not provided, worker nodes will be automatically selected based on logical resource requests. To use a head-node only cluster, pass `[]` here."
        },
    )

    def _validate_worker_nodes(
        self, worker_nodes: Optional[List[Union[WorkerNodeGroupConfig, Dict]]]
    ) -> Optional[List[WorkerNodeGroupConfig]]:
        if worker_nodes is None:
            return None

        if not isinstance(worker_nodes, list) or not all(
            isinstance(c, (dict, WorkerNodeGroupConfig)) for c in worker_nodes
        ):
            raise TypeError(
                "'worker_nodes' must be a list of WorkerNodeGroupConfigs or corresponding dicts"
            )

        duplicate_names = set()
        name_counts: DefaultDict[str, int] = defaultdict(int)
        worker_node_models: List[WorkerNodeGroupConfig] = []
        for node in worker_nodes:
            if isinstance(node, dict):
                node = WorkerNodeGroupConfig.from_dict(node)

            assert isinstance(node, WorkerNodeGroupConfig)
            worker_node_models.append(node)
            name = node.name
            assert name is not None
            name_counts[name] += 1
            if name_counts[name] > 1:
                duplicate_names.add(name)

        if duplicate_names:
            raise ValueError(
                f"'worker_nodes' names must be unique, but got duplicate names: {duplicate_names}"
            )

        return worker_node_models

    min_resources: Optional[ResourceDict] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Total minimum logical resources across all nodes in the cluster. Resources omitted from this field have no minimum.",
            "customer_hosted_only": False,
        },
    )

    def _validate_min_resources(self, min_resources: Optional[ResourceDict]):
        _validate_resource_dict(min_resources, field_name="min_resources")

    max_resources: Optional[ResourceDict] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Total maximum logical resources across all nodes in the cluster. Resources omitted from this field have no maximum.",
            "customer_hosted_only": False,
        },
    )

    def _validate_max_resources(self, max_resources: Optional[ResourceDict]):
        _validate_resource_dict(max_resources, field_name="max_resources")

    zones: Optional[List[str]] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Availability zones to consider for this cluster. Defaults to all zones in the cloud's region. By default all instances with user workloads scheduled on them will run in the same zone to save cost, unless `enable_cross_zone_scaling` is set.",
            "customer_hosted_only": True,
        },
    )

    def _validate_zones(self, zones: Optional[List[str]]):
        if zones is None:
            return
        if not isinstance(zones, list) or not all(isinstance(z, str) for z in zones):
            raise TypeError("'zones' must be a List[str]")
        if len(zones) == 0:
            raise ValueError(
                "'zones' must not be an empty list. Set `None` to default to all zones."
            )

    enable_cross_zone_scaling: bool = field(
        default=False,
        repr=False,
        metadata={
            "docstring": "Allow instances in the cluster to be run across multiple zones. This is recommended when running production services (for fault-tolerance in a zone failure scenario). It is not recommended for workloads that have a large amount of inter-zone communication due to the possibility of higher costs and degraded performance. When false, all instances with user workloads scheduled on them (e.g. all worker nodes in multi-node clusters) will run in the same zone to save cost.",
            "customer_hosted_only": True,
        },
    )

    def _validate_enable_cross_zone_scaling(self, enable_cross_zone_scaling: bool):
        if not isinstance(enable_cross_zone_scaling, bool):
            raise TypeError("'enable_cross_zone_scaling' must be a boolean")

    advanced_instance_config: Optional[AdvancedInstanceConfigDict] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Advanced instance configurations that will be passed through to the cloud provider.",
            "customer_hosted_only": True,
        },
    )

    def _validate_advanced_instance_config(
        self, advanced_instance_config: Optional[AdvancedInstanceConfigDict],
    ):
        _validate_advanced_instance_config_dict(advanced_instance_config)

    flags: Optional[Dict[str, Any]] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Cluster-level flags specifying advanced or experimental options.",
            "customer_hosted_only": False,
        },
    )

    def _validate_flags(self, flags: Optional[Dict[str, Any]]):
        if flags is None:
            return

        if not isinstance(flags, dict):
            raise TypeError("'flags' must be a dict")


@dataclass(frozen=True)
class ComputeConfigVersion(ModelBase):
    """Details of a created version of a compute config.

    Includes the config options and metadata such as the name, version, and ID.
    """

    __doc_py_example__ = """
import anyscale
from anyscale.compute_config.models import (
    ComputeConfigVersion
)

version: ComputeConfigVersion = anyscale.compute_config.get("my-compute-config")
"""

    __doc_cli_example__ = """\
$ anyscale compute-config get -n my-compute-config
name: my-compute-config:1
id: cpt_r4b4b3621rl3uggg7llj3mvme6
config:
  cloud: my-cloud
  head_node:
    instance_type: m5.8xlarge
  worker_nodes:
  - instance_type: m5.8xlarge
    min_nodes: 5
    max_nodes: 5
  - instance_type: m5.4xlarge
    min_nodes: 1
    max_nodes: 10
    market_type: SPOT
"""

    name: str = field(
        metadata={
            "docstring": "Name of the compute config including the version tag, i.e., 'name:version'."
        }
    )

    def _validate_name(self, name: str):
        if not isinstance(name, str):
            raise TypeError("'name' must be a string.")

        if not name.count(":") == 1:
            raise ValueError("'name' must be in the format: '<name>:<version>'.")

    id: str = field(metadata={"docstring": "Unique ID of the compute config."})

    def _validate_id(self, id: str):  # noqa: A002
        if not isinstance(id, str):
            raise TypeError("'id' must be a string.")

    config: ComputeConfig = field(metadata={"docstring": "The compute configuration."},)

    def _validate_config(self, config: ComputeConfig):
        if not isinstance(config, ComputeConfig):
            raise TypeError("'config' must be a ComputeConfig")



# coding: utf-8

"""
    Anyscale API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from anyscale_client.configuration import Configuration


class ClusterComputesQuery(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'project_id': 'str',
        'creator_id': 'str',
        'name': 'TextQuery',
        'include_anonymous': 'bool',
        'paging': 'PageQuery',
        'cloud_id': 'str',
        'version': 'int'
    }

    attribute_map = {
        'project_id': 'project_id',
        'creator_id': 'creator_id',
        'name': 'name',
        'include_anonymous': 'include_anonymous',
        'paging': 'paging',
        'cloud_id': 'cloud_id',
        'version': 'version'
    }

    def __init__(self, project_id=None, creator_id=None, name=None, include_anonymous=False, paging=None, cloud_id=None, version=None, local_vars_configuration=None):  # noqa: E501
        """ClusterComputesQuery - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._project_id = None
        self._creator_id = None
        self._name = None
        self._include_anonymous = None
        self._paging = None
        self._cloud_id = None
        self._version = None
        self.discriminator = None

        if project_id is not None:
            self.project_id = project_id
        if creator_id is not None:
            self.creator_id = creator_id
        if name is not None:
            self.name = name
        if include_anonymous is not None:
            self.include_anonymous = include_anonymous
        if paging is not None:
            self.paging = paging
        if cloud_id is not None:
            self.cloud_id = cloud_id
        if version is not None:
            self.version = version

    @property
    def project_id(self):
        """Gets the project_id of this ClusterComputesQuery.  # noqa: E501

        Filters Cluster Computes by project. If this field is absent, no filtering is done.  # noqa: E501

        :return: The project_id of this ClusterComputesQuery.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this ClusterComputesQuery.

        Filters Cluster Computes by project. If this field is absent, no filtering is done.  # noqa: E501

        :param project_id: The project_id of this ClusterComputesQuery.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def creator_id(self):
        """Gets the creator_id of this ClusterComputesQuery.  # noqa: E501

        Filters Cluster Computes by creator. If this field is absent, no filtering is done.  # noqa: E501

        :return: The creator_id of this ClusterComputesQuery.  # noqa: E501
        :rtype: str
        """
        return self._creator_id

    @creator_id.setter
    def creator_id(self, creator_id):
        """Sets the creator_id of this ClusterComputesQuery.

        Filters Cluster Computes by creator. If this field is absent, no filtering is done.  # noqa: E501

        :param creator_id: The creator_id of this ClusterComputesQuery.  # noqa: E501
        :type: str
        """

        self._creator_id = creator_id

    @property
    def name(self):
        """Gets the name of this ClusterComputesQuery.  # noqa: E501

        Filters Cluster Computes by name. If this field is absent, no filtering is done.  # noqa: E501

        :return: The name of this ClusterComputesQuery.  # noqa: E501
        :rtype: TextQuery
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ClusterComputesQuery.

        Filters Cluster Computes by name. If this field is absent, no filtering is done.  # noqa: E501

        :param name: The name of this ClusterComputesQuery.  # noqa: E501
        :type: TextQuery
        """

        self._name = name

    @property
    def include_anonymous(self):
        """Gets the include_anonymous of this ClusterComputesQuery.  # noqa: E501

        Whether to include anonymous Cluster Computes in the search.  # noqa: E501

        :return: The include_anonymous of this ClusterComputesQuery.  # noqa: E501
        :rtype: bool
        """
        return self._include_anonymous

    @include_anonymous.setter
    def include_anonymous(self, include_anonymous):
        """Sets the include_anonymous of this ClusterComputesQuery.

        Whether to include anonymous Cluster Computes in the search.  # noqa: E501

        :param include_anonymous: The include_anonymous of this ClusterComputesQuery.  # noqa: E501
        :type: bool
        """

        self._include_anonymous = include_anonymous

    @property
    def paging(self):
        """Gets the paging of this ClusterComputesQuery.  # noqa: E501

        Pagination information.  # noqa: E501

        :return: The paging of this ClusterComputesQuery.  # noqa: E501
        :rtype: PageQuery
        """
        return self._paging

    @paging.setter
    def paging(self, paging):
        """Sets the paging of this ClusterComputesQuery.

        Pagination information.  # noqa: E501

        :param paging: The paging of this ClusterComputesQuery.  # noqa: E501
        :type: PageQuery
        """

        self._paging = paging

    @property
    def cloud_id(self):
        """Gets the cloud_id of this ClusterComputesQuery.  # noqa: E501

        Filters Compute Computes by cloud. If this field is absent, no filtering is done.  # noqa: E501

        :return: The cloud_id of this ClusterComputesQuery.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this ClusterComputesQuery.

        Filters Compute Computes by cloud. If this field is absent, no filtering is done.  # noqa: E501

        :param cloud_id: The cloud_id of this ClusterComputesQuery.  # noqa: E501
        :type: str
        """

        self._cloud_id = cloud_id

    @property
    def version(self):
        """Gets the version of this ClusterComputesQuery.  # noqa: E501

        Filters Cluster Computes by version. Versions are positive integers. Setting this field to -1 will return only the latest version of each Cluster Compute. Setting this field to -2 will not filter by version. For example, this can be used to fetch all versions of a Cluster Compute. Deprecated behavior: Setting version to None is equivalent to setting version to '-1'.  # noqa: E501

        :return: The version of this ClusterComputesQuery.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ClusterComputesQuery.

        Filters Cluster Computes by version. Versions are positive integers. Setting this field to -1 will return only the latest version of each Cluster Compute. Setting this field to -2 will not filter by version. For example, this can be used to fetch all versions of a Cluster Compute. Deprecated behavior: Setting version to None is equivalent to setting version to '-1'.  # noqa: E501

        :param version: The version of this ClusterComputesQuery.  # noqa: E501
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                version is not None and version < -2):  # noqa: E501
            raise ValueError("Invalid value for `version`, must be a value greater than or equal to `-2`")  # noqa: E501

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ClusterComputesQuery):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ClusterComputesQuery):
            return True

        return self.to_dict() != other.to_dict()



from dataclasses import dataclass, field
from typing import Any, Dict, Optional

from anyscale._private.models.model_base import ModelBase, ModelEnum


class FineTuningType(ModelEnum):
    LORA = "LORA"
    FULL_PARAM = "FULL_PARAM"

    __docstrings__ = {
        LORA: "Low-Rank Adaptation (LoRA) fine-tuning method.",
        FULL_PARAM: "Full parameter fine-tuning method.",
    }


@dataclass(frozen=True)
class FineTunedModel(ModelBase):
    """Represents a fine-tuned model with its associated metadata."""

    __doc_py_example__ = """\
import anyscale
from anyscale.llm.model.models import FineTunedModel
model: FineTunedModel = anyscale.llm.model.get(model_id="my-model-id")
"""

    __doc_cli_example__ = """\
$ anyscale llm model get --model-id my-model-id
id: my-model-id
base_model_id: meta-llama/Llama-3-8B-Instruct
cloud_id: cloud_abc123
created_at: 1725473924
job_id: prodjob_xyz789
ft_type: LORA
...
"""

    id: str = field(metadata={"docstring": "Unique ID/tag for the fine-tuned model."})

    def _validate_id(self, id: str):  # noqa: A002
        if not isinstance(id, str):
            raise TypeError("'id' must be a string.")

    base_model_id: str = field(
        metadata={"docstring": "Base model ID used for fine-tuning."}
    )

    def _validate_base_model_id(self, base_model_id: str):
        if not isinstance(base_model_id, str):
            raise TypeError("'base_model_id' must be a string.")

    cloud_id: str = field(
        metadata={
            "docstring": "ID for the Anyscale Cloud corresponding to the fine-tuning run."
        }
    )

    def _validate_cloud_id(self, cloud_id: str):
        if not isinstance(cloud_id, str):
            raise TypeError("'cloud_id' must be a string.")

    created_at: int = field(
        metadata={"docstring": "Time at which the fine-tuned model was created."}
    )

    def _validate_created_at(self, created_at: int):
        if not isinstance(created_at, int):
            raise TypeError("'created_at' must be an integer (Unix timestamp).")

        if created_at < 0:
            raise ValueError("'created_at' must be a positive integer")

    creator: Optional[str] = field(
        metadata={"docstring": "Email address for the user who created the model."}
    )

    def _validate_creator(self, creator: Optional[str]):
        if creator is not None and not isinstance(creator, str):
            raise TypeError("'creator' must be a string or None.")

    ft_type: FineTuningType = field(metadata={"docstring": "Fine-tuning type."})

    def _validate_ft_type(self, ft_type: FineTuningType) -> FineTuningType:
        return FineTuningType.validate(ft_type)

    generation_config: Optional[Dict[str, Any]] = field(
        metadata={
            "docstring": "Inference generation config with chat-templating parameters and stopping sequences."
        }
    )

    def _validate_generation_config(self, generation_config: Optional[Dict[str, Any]]):
        if generation_config is not None and not isinstance(generation_config, dict):
            raise TypeError("'generation_config' must be a dictionary or None.")

    job_id: Optional[str] = field(
        metadata={
            "docstring": "ID for the Anyscale job corresponding to the fine-tuning run, if applicable."
        }
    )

    def _validate_job_id(self, job_id: Optional[str]):
        if job_id is not None and not isinstance(job_id, str):
            raise TypeError("'job_id' must be a string or None.")

    project_id: Optional[str] = field(
        metadata={
            "docstring": "ID for the Anyscale Project corresponding to the fine-tuning run."
        }
    )

    def _validate_project_id(self, project_id: Optional[str]):
        if project_id is not None and not isinstance(project_id, str):
            raise TypeError("'project_id' must be a string.")

    storage_uri: str = field(
        metadata={
            "docstring": "URI at which the fine-tuned model checkpoint is stored."
        }
    )

    def _validate_storage_uri(self, storage_uri: str):
        if not isinstance(storage_uri, str):
            raise TypeError("'storage_uri' must be a string.")

    workspace_id: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "ID for the Anyscale Workspace in which the model was fine-tuned, if any."
        },
    )

    def _validate_workspace_id(self, workspace_id: Optional[str]):
        if workspace_id is not None and not isinstance(workspace_id, str):
            raise TypeError("'workspace_id' must be a string or None.")


@dataclass(frozen=True)
class DeletedFineTunedModel(ModelBase):
    """Represents a deleted fine-tuned model with its deletion metadata."""

    __doc_py_example__ = """\
import anyscale
from anyscale.llm.model.models import DeletedFineTunedModel
deleted_model: DeletedFineTunedModel = anyscale.llm.model.delete("my-model-id")
"""

    __doc_cli_example__ = """\
$ anyscale models get-deleted my-model-id
id: my-model-id
deleted_at: 1725473924
"""

    id: str = field(metadata={"docstring": "Unique ID/tag for the fine-tuned model."})

    def _validate_id(self, id: str):  # noqa: A002
        if not isinstance(id, str):
            raise TypeError("'id' must be a string.")

    deleted_at: int = field(
        metadata={
            "docstring": "Unix timestamp (in seconds) at which the fine-tuned model was deleted."
        }
    )

    def _validate_deleted_at(self, deleted_at: int):
        if not isinstance(deleted_at, int):
            raise TypeError("'deleted_at' must be an integer (Unix timestamp).")

        if deleted_at < 0:
            raise ValueError("'deleted_at' must be a positive integer (Unix timestamp)")



GET_PY_EXAMPLE = """
import anyscale
from anyscale.llm.dataset import Dataset

dataset: Dataset = anyscale.llm.dataset.get("my_first_dataset")
print(f"Dataset name: '{dataset.name}'")  # Dataset name: 'my_first_dataset'

# Get the second latest version of the dataset
prev_dataset = anyscale.llm.dataset.get("my_first_dataset", version=-1)
"""

GET_PY_ARG_DOCSTRINGS = {
    "name": "Name of the dataset",
    "version": "Version of the dataset. If a negative integer is provided, the dataset returned is this many versions back of the latest version. Default: Latest version.",
    "project": "Name of the Anyscale project that the dataset belongs to. If not provided, all projects will be searched.",
}

UPLOAD_PY_EXAMPLE = """
import anyscale

anyscale.llm.dataset.upload("path/to/my_first_dataset.jsonl", name="my_first_dataset")
anyscale.llm.dataset.upload("my_dataset.jsonl", "second_dataset")
anyscale.llm.dataset.upload("my_dataset2.jsonl", "second_dataset", description="added 3 lines")
"""

UPLOAD_PY_ARG_DOCSTRINGS = {
    "dataset_file": "Path to the dataset file to upload.",
    "name": "Name of a new dataset, or an existing dataset, to upload a new version of.",
    "description": "Description of the dataset version.",
    "cloud": "Name of the Anyscale cloud to upload a new dataset to. If not provided, the default cloud will be used.",
    "project": "Name of the Anyscale project to upload a new dataset to. If not provided, the default project of the cloud will be used.",
}

DOWNLOAD_PY_EXAMPLE = """
import anyscale

dataset_contents: bytes = anyscale.llm.dataset.download("my_first_dataset.jsonl")
jsonl_obj = [json.loads(line) for line in dataset_contents.decode().splitlines()]

prev_dataset_contents = anyscale.llm.dataset.download("my_first_dataset.jsonl", version=-1)
"""

DOWNLOAD_PY_ARG_DOCSTRINGS = {
    "name": "Name of the dataset to download.",
    "version": "Version of the dataset to download. If a negative integer is provided, the dataset returned is this many versions back of the latest version. Default: Latest version.",
    "project": "Name of the Anyscale project to download the dataset from. If not provided, all projects will be searched.",
}



import json
import os
import sys
from typing import Any, Tuple

import click
import requests

from anyscale.authenticate import get_auth_api_client
from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client.models.execute_shell_command_options import (
    ExecuteShellCommandOptions,
)
from anyscale.commands import workspace_commands_v2
from anyscale.commands.util import LegacyAnyscaleCommand
from anyscale.controllers.cluster_controller import ClusterController
from anyscale.controllers.workspace_controller import WorkspaceController
from anyscale.project import find_project_root
from anyscale.shared_anyscale_utils.utils.byod import BYODInfo
from anyscale.util import get_endpoint
from anyscale.workspace_utils import (
    get_workspace_root_or_throw,
    write_workspace_id_to_disk,
)


log = BlockLogger()  # CLI Logger


@click.group("workspace", help="Interact with workspaces on Anyscale.")
def workspace_cli() -> None:
    pass


@workspace_cli.command(
    name="create",
    help="Create a workspace on Anyscale.",
    cls=LegacyAnyscaleCommand,
    new_prefix="anyscale workspace_v2",
    new_cli=workspace_commands_v2.create,
)
@click.option(
    "--name", "-n", required=True, help="Name of the workspace to create.",
)
@click.option(
    "--project-id",
    required=False,
    help="ID of the project to associate with the workspace.",
)
@click.option(
    "--cloud-id", required=True, help="ID of the cloud to use for the workspace.",
)  # Note: we could further improve UX by making cloud_id optional since we can always infer it from compute config
@click.option(
    "--cluster-env-build-id",
    required=False,
    help="ID of the cluster environment build to use for the workspace.",
)
@click.option(
    "--docker", required=False, default=None, help=("Custom docker image URI."),
)
@click.option(
    "--python-version",
    required=False,
    default=None,
    help=("Python version for the custom docker image."),
)
@click.option(
    "--ray-version",
    required=False,
    default=None,
    help=("Ray version for the custom docker image."),
)
@click.option(
    "--compute-config-id",
    required=True,
    help="ID of the compute config to use for the workspace.",
)
def create(  # noqa: PLR0913
    name: str,
    project_id: str,
    cloud_id: str,
    cluster_env_build_id: str,
    docker: str,
    python_version: str,
    ray_version: str,
    compute_config_id: str,
) -> None:
    if cluster_env_build_id is None and docker is None:
        raise click.ClickException(
            "Please specify one of `--docker` or `--cluster-env-build-id`."
        )
    if docker is not None:
        if cluster_env_build_id is not None:
            raise click.ClickException(
                "`--cluster-env-build-id` and `--docker` cannot both be "
                "specified. Please only provide one of these two arguments."
            )

        if python_version is None:
            raise click.ClickException(
                "`--python-version` should be specified when `--docker` is used."
            )
        if ray_version is None:
            raise click.ClickException(
                "`--ray-version` should be specified when `--docker` is used."
            )
        # Create docker build_id
        cluster_env_build_id = BYODInfo(docker, python_version, ray_version).encode()

    workspace_controller = WorkspaceController()
    workspace_controller.create(
        name=name,
        cloud_id=cloud_id,
        compute_config_id=compute_config_id,
        project_id=project_id,
        cluster_environment_build_id=cluster_env_build_id,
    )

    log.info(f"Workspace {name} created successfully.")


@workspace_cli.command(
    name="start",
    help="Start an existing workspace on Anyscale.",
    cls=LegacyAnyscaleCommand,
    new_prefix="anyscale workspace_v2",
    new_cli=workspace_commands_v2.start,
)
@click.option(
    "--name", "-n", required=True, help="Name of existing workspace to start.",
)
def start(name: str) -> None:
    cluster_controller = ClusterController()

    workspace = get_workspace_from_name(name)
    cluster_id = workspace.cluster_id

    cluster_controller.start(
        cluster_name=None,
        cluster_id=cluster_id,
        cluster_env_name=None,
        docker=None,
        python_version=None,
        ray_version=None,
        cluster_compute_name=None,
        cluster_compute_file=None,
        cloud_name=None,
        idle_timeout=None,
        project_id=None,
        project_name=None,
        user_service_access=None,
    )


@workspace_cli.command(
    name="terminate",
    help="Terminate a workspace on Anyscale.",
    cls=LegacyAnyscaleCommand,
    new_prefix="anyscale workspace_v2",
    new_cli=workspace_commands_v2.terminate,
)
@click.option(
    "--name", "-n", required=True, help="Name of existing workspace to terminate.",
)
def terminate(name: str) -> None:
    cluster_controller = ClusterController()
    workspace = get_workspace_from_name(name)
    cluster_id = workspace.cluster_id

    cluster_controller.terminate(
        cluster_name=None,
        cluster_id=cluster_id,
        project_id=None,
        project_name=None,
        cloud_id=None,
        cloud_name=None,
    )


@workspace_cli.command(
    name="clone",
    help="Clone a workspace on Anyscale.",
    cls=LegacyAnyscaleCommand,
    is_limited_support=True,
)
@click.option(
    "--name", "-n", required=True, help="Name of existing workspace to clone.",
)
def clone(name: str) -> None:
    """
    Clone the workspace to a local dir whose name is the name of the workspace.
    """
    _check_local()

    workspace = get_workspace_from_name(name)
    dest = workspace.name
    workspace_controller = WorkspaceController()
    if os.path.exists(dest):
        _exit_error(
            f"Cannot clone workspace: already cloned locally at '{os.path.abspath(dest)}'."
        )
    workspace_controller.clone(workspace)
    os.chdir(dest)
    _do_pull(pull_git_state=True)


@workspace_cli.command(name="activate")
@click.argument(
    "name", required=True, default=None,
)
def activate(name: str) -> None:
    """Activate a workspace.

    If the current directory is already a part of a workspace, change the workspace.
    Else, setup a new workspace rooted at the current directory

    Args:
        name: Name of the workspace to activate.
    """
    _check_local()
    root_dir = find_project_root(os.getcwd())
    if not root_dir:
        raise click.ClickException(
            "Could not find the root workspace directory. Please first run `anyscale workspace clone`"
        )
    try:
        workspace = get_workspace_from_name(name)
        workspace_id = workspace.id
    except Exception:  # noqa: BLE001
        workpaces_url = get_endpoint("/workspaces")
        raise click.ClickException(
            f"There is no workspace {name} registered with Anyscale. You can view your workspaces here: {workpaces_url}"
        )
    write_workspace_id_to_disk(workspace_id, root_dir)


@workspace_cli.command(
    name="pull",
    help="Pull files from a workspace on Anyscale.",
    cls=LegacyAnyscaleCommand,
    new_prefix="anyscale workspace_v2",
    new_cli=workspace_commands_v2.pull,
)
@click.option(
    "--pull-git-state",
    required=False,
    is_flag=True,
    default=False,
    help="Also pull git state. This will add additional overhead.",
)
def pull(pull_git_state) -> None:
    _check_local()
    _check_workspace()
    _do_pull(pull_git_state)


@workspace_cli.command(
    name="push",
    help="Push files to a workspace on Anyscale.",
    cls=LegacyAnyscaleCommand,
    new_prefix="anyscale workspace_v2",
    new_cli=workspace_commands_v2.push,
)
@click.option(
    "--push-git-state",
    required=False,
    is_flag=True,
    default=False,
    help="Also push git state. This is currently unoptimized and will be very slow.",
)
def push(push_git_state) -> None:
    _check_local()
    _check_workspace()
    _do_push(push_git_state)


@workspace_cli.command(
    name="run",
    help="Run a command in a workspace, syncing files first if needed.",
    cls=LegacyAnyscaleCommand,
    new_prefix="anyscale workspace_v2",
    new_cli=workspace_commands_v2.run_command,
)
@click.argument("command", required=True)
@click.option(
    "--web-terminal",
    "-w",
    required=False,
    is_flag=True,
    default=False,
    help="Run the command in the webterminal. Progress can be tracked from the UI.",
)
@click.option(
    "--as-job",
    "-j",
    required=False,
    is_flag=True,
    default=False,
    help="Run the command as a background job in a new cluster.",
)
@click.option(
    "--no-push",
    "-s",
    required=False,
    is_flag=True,
    default=False,
    help="Whether to skip pushing files prior to running the command.",
)
def run(command: str, web_terminal: bool, as_job: bool, no_push: bool,) -> None:
    _check_local()
    _check_workspace()
    if as_job:
        raise NotImplementedError("Running as a job isn't implemented yet.")
    workspace_controller = WorkspaceController()
    # Generally, we assume the user wants to run their command in the context of
    # their latest file changes.
    if not no_push:
        _do_push(push_git_state=False)
    workspace_controller = WorkspaceController()
    dir_name = workspace_controller.get_workspace_dir_name()
    if web_terminal:
        cluster_id = workspace_controller.get_activated_workspace().cluster_id
        _execute_shell_command(cluster_id, f"cd ~/{dir_name} && {command}")
        # TODO(ekl) show the workspace URL here and also block on completion.
        print()
        print(
            "Command submitted succcessfully! See the 'Command History' tab "
            "of this workspace to view command status and output."
        )
        print()
    else:
        workspace_controller.run_cmd(cmd=f"cd ~/{dir_name} && {command}", args=[])


@workspace_cli.command(
    name="ssh",
    help="ssh into a workspace, you can also pass args to the ssh command. E.g. 'anyscale workspace ssh -- -L 8888:localhost:8888",
    cls=LegacyAnyscaleCommand,
    new_cli=workspace_commands_v2.ssh,
    new_prefix="anyscale workspace_v2",
)
@click.argument(
    "args", nargs=-1, required=False, type=click.UNPROCESSED,
)
def ssh(args: Tuple[str]) -> None:
    """
    ssh into a running workspace.
    """
    _check_local()

    workspace_controller = WorkspaceController()
    dir_name = workspace_controller.get_workspace_dir_name()
    workspace_controller.run_cmd(f"cd ~/{dir_name} && /bin/bash", args=list(args))


@workspace_cli.command(
    name="list", help="prints information about existing workspaces", hidden=True,
)
def list_command() -> None:
    workspace_controller = WorkspaceController()
    workspace_controller.list()


@workspace_cli.command(name="cp", cls=LegacyAnyscaleCommand, is_limited_support=True)
@click.argument(
    "remote_path",
    nargs=1,
    required=True,
    type=click.Path(
        readable=False  # we don't want to check the readability of a remote path against local file
    ),
)
@click.argument(
    "local_path", nargs=1, required=True, type=click.Path(writable=True),
)
def copy_command(remote_path, local_path) -> None:
    """
        Copy a file or a directory from workspace to local file system.

        Examples

            anyscale workspace cp /mnt/shared_objects/foo.py /tmp/copy_of_foo.py

            anyscale workspace cp "~/default/README.md" ~/Downlaods

            anyscale workspace cp "/tmp/" ~/Downlaods

    """

    _do_copy(remote_path, local_path)


def _do_pull(pull_git_state):
    workspace_controller = WorkspaceController()
    dir_name = workspace_controller.get_workspace_dir_name()
    # Since workspaces store git objects in an EFS alternates dir, we have to force
    # a repack prior to pulling. Otherwise, the pulled git repo may not be fully
    # functional locally. A repack is expensive, but we assume pulls aren't frequent.
    if pull_git_state:
        workspace_controller.run_cmd(
            f"cd ~/{dir_name} && python -m snapshot_util repack_git_repos", args=[],
        )

    workspace_root = get_workspace_root_or_throw()
    workspace_controller.run_rsync(
        local_path=workspace_root,
        down=True,
        rsync_filters=[".gitignore"],
        rsync_excludes=[".anyscale.yaml", ".git/objects/info/alternates"]
        + ([] if pull_git_state else [".git"]),
        delete_existing_files_in_destination=True,
    )


def _do_push(push_git_state):
    workspace_controller = WorkspaceController()
    workspace_root = get_workspace_root_or_throw()
    workspace_controller.run_rsync(
        local_path=workspace_root,
        down=False,
        rsync_filters=[".gitignore"],
        # TODO(ekl) to efficiently push the git state, we need to do this in two
        # phases: first sync the shared git objects to EFS, then sync non .git files.
        # Otherwise this will be very slow since our local git representation is
        # different from the remote one (has large .pack files).
        rsync_excludes=[".anyscale.yaml", ".git/objects/info/alternates"]
        + ([] if push_git_state else [".git"]),
        delete_existing_files_in_destination=True,
    )


def _do_copy(remote_path: str, local_path: str):
    workspace_controller = WorkspaceController()
    workspace_controller.run_rsync(
        local_path=local_path,
        remote_path=remote_path,
        down=True,
        rsync_filters=[],
        rsync_excludes=[],
    )


def _check_local():
    if "ANYSCALE_WORKING_DIR" in os.environ:
        _exit_error(
            "Error: This command cannot be run from inside an Anyscale cluster."
        )


def _check_workspace():
    if not os.path.exists(".anyscale.yaml"):
        _exit_error(
            "Error: This command must be run from the root of a cloned workspace directory."
        )


def get_workspace_from_name(name: str) -> Any:
    """Get a workspace from its name."""

    # Find the workspace by name
    auth_api_client = get_auth_api_client()
    results = auth_api_client.api_client.list_workspaces_api_v2_experimental_workspaces_get(
        name=name
    ).results
    if len(results) == 0:
        _exit_error(f"No workspace with name {name} found.")
    elif len(results) > 1:
        _exit_error(f"Multiple workspaces with name {name} found.")
    return results[0]


def _exit_error(msg: str) -> None:
    print()
    print(msg)
    sys.exit(1)


def _execute_shell_command(cluster_id: str, shell_command: str):
    auth_api_client = get_auth_api_client()

    result = auth_api_client.api_client.execute_interactive_command_api_v2_sessions_session_id_execute_interactive_command_post(
        cluster_id, ExecuteShellCommandOptions(shell_command=shell_command),
    ).result

    command_id = result.command_id

    body = {
        "command_id": command_id,
        "shell_command": shell_command,
    }

    body_string = json.dumps(body)

    cluster = auth_api_client.api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get(
        cluster_id
    ).result

    webterminal_auth_url = cluster.webterminal_auth_url
    index = webterminal_auth_url.index("/auth/") if webterminal_auth_url else None
    cluster_dns_name = webterminal_auth_url[:index] if index else None
    if not cluster_dns_name:
        return

    with requests.Session() as session:
        session.get(webterminal_auth_url)
        web_terminals_endpoint = f"{cluster_dns_name}/webterminal/exec"
        session.post(web_terminals_endpoint, data=body_string)



from io import StringIO
from typing import IO, Optional

import click
import yaml

import anyscale
from anyscale.commands import command_examples
from anyscale.commands.util import AnyscaleCommand


@click.group(
    "image", help="Manage images to define dependencies on Anyscale.",
)
def image_cli() -> None:
    pass


@image_cli.command(
    name="build",
    help=("Build an image from a Containerfile."),
    cls=AnyscaleCommand,
    example=command_examples.IMAGE_BUILD_EXAMPLE,
)
@click.option(
    "--containerfile",
    "-f",
    help="Path to the Containerfile.",
    type=click.File("rb"),
    required=True,
)
@click.option(
    "--name",
    "-n",
    help="Name for the image. If the image with the same name already exists, a new version will be built. Otherwise, a new image will be created.",
    required=True,
    type=str,
)
@click.option(
    "--ray-version",
    "-r",
    help="The Ray version (X.Y.Z) specified for this image specified by either an image URI or a containerfile. If not provided, the latest Ray version will be used.",
    type=str,
    default=None,
)
def build(
    containerfile: IO[bytes], name: str, ray_version: Optional[str] = None
) -> None:
    containerfile_str = containerfile.read().decode("utf-8")
    image_uri = anyscale.image.build(
        containerfile_str, name=name, ray_version=ray_version
    )
    print(f"Image built successfully with URI: {image_uri}")


@image_cli.command(
    name="get",
    help=("Get details of an image."),
    cls=AnyscaleCommand,
    example=command_examples.IMAGE_GET_EXAMPLE,
)
@click.option(
    "--name",
    "-n",
    help=(
        "Get the details of an image.\n\n"
        "The name can contain an optional version, e.g., 'name:version'. "
        "If no version is provided, the latest one will be used.\n\n"
    ),
    type=str,
    default=None,
    required=True,
)
def get(name: str) -> None:
    image_build = anyscale.image.get(name=name)
    stream = StringIO()
    yaml.safe_dump(image_build.to_dict(), stream, sort_keys=False)
    print(stream.getvalue(), end="")


@image_cli.command(
    name="register",
    help=("Register a custom container image with a container image name."),
    cls=AnyscaleCommand,
    example=command_examples.IMAGE_REGISTER_EXAMPLE,
)
@click.option(
    "--image-uri",
    help="The URI of the custom container image to register.",
    type=str,
    required=True,
)
@click.option(
    "--name",
    "-n",
    help="Name for the container image. If the name already exists, a new version will be built. Otherwise, a new container image will be created.",
    required=True,
    type=str,
)
@click.option(
    "--ray-version",
    "-r",
    help="The Ray version (X.Y.Z) specified for this image specified by either an image URI or a containerfile. If not provided, the latest Ray version will be used.",
    type=str,
    default=None,
)
@click.option(
    "--registry-login-secret",
    help="Name or identifier of the secret containing credentials to authenticate to the docker registry hosting the image.",
    type=str,
    default=None,
)
def register(
    image_uri: str,
    name: str,
    ray_version: Optional[str] = None,
    registry_login_secret: Optional[str] = None,
) -> None:
    built_image_uri = anyscale.image.register(
        image_uri=image_uri,
        registry_login_secret=registry_login_secret,
        ray_version=ray_version,
        name=name,
    )
    print(f"Image registered successfully with URI: {built_image_uri}")



JOB_SUBMIT_EXAMPLE = """\
$ anyscale job submit --name my-job --wait -- python main.py
Output
(anyscale +1.0s) Submitting job with config JobConfig(name='my-job', image_uri=None, compute_config=None, env_vars=None, py_modules=None, cloud=None, project=None, ray_version=None, job_queue_config=None).
(anyscale +1.7s) Uploading local dir '.' to cloud storage.
(anyscale +2.6s) Including workspace-managed pip dependencies.
(anyscale +3.2s) Job 'my-job' submitted, ID: 'prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe'.
(anyscale +3.2s) View the job in the UI: https://console.anyscale.com/jobs/prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe
(anyscale +3.2s) Waiting for the job to run. Interrupting this command will not cancel the job.
(anyscale +3.5s) Waiting for job 'prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe' to reach target state SUCCEEDED, currently in state: STARTING
(anyscale +1m19.7s) Job 'prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe' transitioned from STARTING to SUCCEEDED
(anyscale +1m19.7s) Job 'prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe' reached target state, exiting
"""

JOB_STATUS_EXAMPLE = """\
$ anyscale job status -n my-job
id: prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe
name: my-job
state: STARTING
runs:
- name: raysubmit_ynxBVGT1SmzndiXL
  state: SUCCEEDED
"""

JOB_TERMINATE_EXAMPLE = """\
$ anyscale job terminate -n my-job
(anyscale +5.4s) Marked job 'my-job' for termination
(anyscale +5.4s) Query the status of the job with `anyscale job status --name my-job`.
"""

JOB_ARCHIVE_EXAMPLE = """\
$ anyscale job archive -n my-job
(anyscale +8.5s) Job prodjob_vzq2pvkzyz3c1jw55kl76h4dk1 is successfully archived.
"""

JOB_LOGS_EXAMPLE = """\
$ anyscale job logs -n my-job
2024-08-23 20:31:10,913 INFO job_manager.py:531 -- Runtime env is setting up.
hello world
"""

JOB_WAIT_EXAMPLE = """\
$ anyscale job wait -n my-job
(anyscale +5.7s) Waiting for job 'my-job' to reach target state SUCCEEDED, currently in state: STARTING
(anyscale +1m34.2s) Job 'my-job' transitioned from STARTING to SUCCEEDED
(anyscale +1m34.2s) Job 'my-job' reached target state, exiting
"""

JOB_LIST_EXAMPLE = """\
$ anyscale job list -n my-job
Output
View your Jobs in the UI at https://console.anyscale.com/jobs
JOBS:
NAME    ID                                    COST  PROJECT NAME    CLUSTER NAME                                    CURRENT STATE           CREATOR           ENTRYPOINT
my-job  prodjob_s9x4uzc5jnkt5z53g4tujb3y2e       0  default         cluster_for_prodjob_s9x4uzc5jnkt5z53g4tujb3y2e  SUCCESS                 doc@anyscale.com  python main.py
"""

SCHEDULE_APPLY_EXAMPLE = """\
$ anyscale schedule apply -n my-schedule -f my-schedule.yaml
(anyscale +0.5s) Applying schedule with config ScheduleConfig(job_config=JobConfig(name='my-schedule', image_uri=None, compute_config=None, env_vars=None, py_modules=None, cloud=None, project=None, ray_version=None, job_queue_config=None), cron_expression='0 0 * * * *', timezone='UTC').
(anyscale +2.3s) Uploading local dir '.' to cloud storage.
(anyscale +3.7s) Including workspace-managed pip dependencies.
(anyscale +4.9s) Schedule 'my-schedule' submitted, ID: 'cronjob_vrjrbwcnfjjid7fsld3sfkn8jz'.

$ cat my-schedule.yaml
timezone: local
cron_expression: 0 0 * * * *
job_config:
    name: my-job
    entrypoint: python main.py
    max_retries: 5
"""

SCHEDULE_LIST_EXAMPLE = """\
$ anyscale schedule list -n my-schedule
Output
+------------------------------------+-------------+---------------+-----------+-------------+------------------+------------+------------------+-----------------------+
| ID                                 | NAME        | DESCRIPTION   | PROJECT   | CRON        | NEXT TRIGGER     | TIMEZONE   | CREATOR          | LATEST EXECUTION ID   |
|------------------------------------+-------------+---------------+-----------+-------------+------------------+------------+------------------+-----------------------|
| cronjob_vrjrbwcnfjjid7fsld3sfkn8jz | my-schedule |               | default   | 0 0 * * * * | 2 hours from now | UTC        | doc@anyscale.com |                       |
+------------------------------------+-------------+---------------+-----------+-------------+------------------+------------+------------------+-----------------------+
"""

SCHEDULE_PAUSE_EXAMPLE = """\
$ anyscale schedule pause -n my-schedule
(anyscale +3.6s) Set schedule 'my-schedule' to state DISABLED
"""

SCHEDULE_RESUME_EXAMPLE = """\
$ anyscale schedule resume -n my-schedule
(anyscale +4.1s) Set schedule 'my-schedule' to state ENABLED
"""

SCHEDULE_STATUS_EXAMPLE = """\
$ anyscale schedule status -n my-schedule
id: cronjob_vrjrbwcnfjjid7fsld3sfkn8jz
name: my-schedule
state: ENABLED
"""

SCHEDULE_RUN_EXAMPLE = """\
$ anyscale schedule run -n my-schedule
(anyscale +2.5s) Triggered job for schedule 'my-schedule'.
"""

SCHEDULE_URL_EXAMPLE = """\
$ anyscale schedule url -n my-schedule
Output
(anyscale +2.3s) View your schedule at https://console.anyscale.com/scheduled-jobs/cronjob_7zj
"""

WORKSPACE_CREATE_EXAMPLE = """\
$ anyscale workspace_v2 create -f config.yml
(anyscale +2.7s) Workspace created successfully id: expwrk_jstjkv15a1vmle2j1t59s4bm35
(anyscale +3.9s) Applied dynamic requirements to workspace id: my-workspace
(anyscale +4.8s) Applied environment variables to workspace id: my-workspace

$ cat config.yml
name: my-workspace
idle_termination_minutes: 10
env_vars:
    HUMPDAY: WEDS
requirements: requirements.txt
"""

WORKSPACE_START_EXAMPLE = """\
$ anyscale workspace_v2 start --name my-workspace
(anyscale +5.8s) Starting workspace 'my-workspace'
"""

WORKSPACE_TERMINATE_EXAMPLE = """\
$ anyscale workspace_v2 terminate --name my-workspace
(anyscale +2.5s) Terminating workspace 'my-workspace'
"""

WORKSPACE_STATUS_EXAMPLE = """\
$ anyscale workspace_v2 status --name my-workspace
(anyscale +2.3s) STARTING
"""

WORKSPACE_WAIT_EXAMPLE = """\
$ anyscale workspace_v2 wait --name my-workspace --state RUNNING
(anyscale +2.5s) Waiting for workspace 'expwrk_jstjkv15a1vmle2j1t59s4bm35' to reach target state RUNNING, currently in state: RUNNING
(anyscale +2.8s) Workspace 'expwrk_jstjkv15a1vmle2j1t59s4bm35' reached target state, exiting
"""

WORKSPACE_SSH_EXAMPLE = """\
$ anyscale workspace_v2 ssh --name my-workspace
Authorized uses only. All activity may be monitored and reported.
Warning: Permanently added '[0.0.0.0]:5020' (ED25519) to the list of known hosts.
(base) ray@ip-10-0-24-60:~/default$
"""

WORKSPACE_RUN_COMMAND_EXAMPLE = """\
$ anyscale workspace_v2 run_command --name my-workspace "echo hello world"
Authorized uses only. All activity may be monitored and reported.
Warning: Permanently added '[0.0.0.0]:5020' (ED25519) to the list of known hosts.
hello world
"""

WORKSPACE_PULL_EXAMPLE = """\
$ anyscale workspace_v2 pull --name my-workspace --local-dir my-local
Warning: Permanently added '54.212.209.253' (ED25519) to the list of known hosts.
Authorized uses only. All activity may be monitored and reported.
Warning: Permanently added '[0.0.0.0]:5020' (ED25519) to the list of known hosts.
receiving incremental file list
created directory my-local
./
my-local/
my-local/main.py

sent 118 bytes  received 141 bytes  172.67 bytes/sec
total size is 0  speedup is 0.00
"""

WORKSPACE_PUSH_EXAMPLE = """\
$ anyscale workspace_v2 push --name my-workspace --local-dir my-local
Warning: Permanently added '52.10.22.124' (ED25519) to the list of known hosts.
Authorized uses only. All activity may be monitored and reported.
Warning: Permanently added '[0.0.0.0]:5020' (ED25519) to the list of known hosts.
sending incremental file list
my-local/
my-local/main.py

sent 188 bytes  received 39 bytes  151.33 bytes/sec
total size is 0  speedup is 0.00
"""

MACHINE_POOL_CREATE_EXAMPLE = """\
$ anyscale machine-pool create --name can-testing
Machine pool can-testing has been created successfully (ID mp_8ogdz85mdwxb8a92yo44nn84ox).
"""

MACHINE_POOL_DELETE_EXAMPLE = """\
$ anyscale machine-pool delete --name can-testing
Deleted machine pool 'can-testing'.
"""

MACHINE_POOL_LIST_EXAMPLE = """\
$ anyscale machine-pool list
MACHINE POOL       ID                             Clouds
can-testing        mp_8ogdz85mdwxb8a92yo44nn84ox
"""

LLM_MODELS_GET_EXAMPLE = """
$ anyscale llm model get --model-id my-model-id
Output
{
    'id': 'my-model-id',
    'base_model_id': 'meta-llama/Meta-Llama-3-8B',
    'storage_uri': 'gs://my_bucket/my_folder',
    'ft_type': 'LORA',
    'cloud_id': 'cld_tffbxe9ia5phqr1unxhz4f7e1e',
    'project_id': 'prj_dqb6ha67zubz3gdlvn2tmmglb8',
    'created_at': 1725563985,
    'creator': 'test@anyscale.com',
    'job_id': 'N/A',
    'workspace_id': 'expwrk_yje3t8twim18iuta9r45gwcgcn',
    'generation_config': {
        'prompt_format': {
            'system': '<|start_header_id|>system<|end_header_id|>\\n\\n{instruction}<|eot_id|>',
            'assistant': '<|start_header_id|>assistant<|end_header_id|>\\n\\n{instruction}<|eot_id|>',
            'trailing_assistant': '<|start_header_id|>assistant<|end_header_id|>\\n\\n',
            'user': '<|start_header_id|>user<|end_header_id|>\\n\\n{instruction}<|eot_id|>',
            'bos': '<|begin_of_text|>',
            'default_system_message': '',
            'add_system_tags_even_if_message_is_empty': False,
            'system_in_user': False,
            'system_in_last_user': False,
            'strip_whitespace': True
        },
        'stopping_sequences': None
    }
}
"""

LLM_MODELS_LIST_EXAMPLE = """
$ anyscale llm model list --cloud-id cld_1j41ls4gwkga4pwp8nbql6f239 --project_id prj_i4wy1t442cbe2sthxp61dmtkbh --max-items 2
Output
[
    {
        'id': 'meta-llama/Meta-Llama-3-8B-Instruct:test:bnkve',
        'base_model_id': 'meta-llama/Meta-Llama-3-8B-Instruct',
        'storage_uri': 's3://anyscale-production-data-cld-1j41ls4gwkga4pwp...',
        'ft_type': 'LORA',
        'cloud_id': 'cld_1j41ls4gwkga4pwp8nbql6f239',
        'project_id': 'prj_i4wy1t442cbe2sthxp61dmtkbh',
        'created_at': 1725572462,
        'creator': 'test@anyscale.com',
        'job_id': 'N/A',
        'workspace_id': 'expwrk_bqld1y579g3clukr49rsnd7i5m',
        'generation_config': '{"prompt_format": {"system": "<|start_header_id|>s...'
    },
    {
        'id': 'neuralmagic/Meta-Llama-3.1-8B-Instruct-FP8:test:czcal',
        'base_model_id': 'neuralmagic/Meta-Llama-3.1-8B-Instruct-FP8',
        'storage_uri': 'gs://storage-bucket-cld-tffbxe9ia5phqr1unxhz4f7e1e...',
        'ft_type': 'LORA',
        'cloud_id': 'cld_1j41ls4gwkga4pwp8nbql6f239',
        'project_id': 'prj_i4wy1t442cbe2sthxp61dmtkbh',
        'created_at': 1725563985,
        'creator': 'test@anyscale.com',
        'job_id': 'N/A',
        'workspace_id': 'expwrk_yje3t8twim18iuta9r45gwcgcn',
        'generation_config': '{"prompt_format": {"system": "<|start_header_id|>s...'
    }
]
"""

LLM_MODELS_DELETE_EXAMPLE = """
$ anyscale llm model delete --model-id my-model-id
Output
{'id': 'my-model-id', 'deleted_at': 1725572462}
"""

LLM_DATASET_GET_EXAMPLE = """
$ anyscale llm dataset get john_doe/viggo/train.jsonl
Dataset(
    id='dataset_123',
    name='john_doe/viggo/train.jsonl',
    filename='train.jsonl',
    storage_uri='s3://anyscale-test-data-cld-123/org_123/cld_123/datasets/dataset_123/3/john_doe/viggo/train.jsonl',
    version=3,
    num_versions=3,
    created_at=datetime.datetime(2024, 1, 1, 0, 0, tzinfo=tzutc()),
    creator_id='usr_123',
    project_id='prj_123',
    cloud_id='cld_123',
    description=None
)
"""

LLM_DATASET_UPLOAD_EXAMPLE = """
$ anyscale llm dataset upload path/to/my_dataset.jsonl -n my_first_dataset

0:00:00 100% ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 5.1 MB / 5.1 MB Uploading '/path/to/my_dataset.jsonl'

Upload complete!

Dataset(
    id='dataset_123',
    name='my_first_dataset',
    filename='my_dataset.jsonl',
    storage_uri='s3://anyscale-test-data-cld-123/org_123/cld_123/datasets/dataset_123/1/my_dataset.jsonl',
    version=1,
    num_versions=1,
    created_at=datetime.datetime(2024, 1, 1, 0, 0, tzinfo=tzutc()),
    creator_id='usr_123',
    project_id='prj_123',
    cloud_id='cld_123',
    description=None
)
"""

LLM_DATASET_DOWNLOAD_EXAMPLE = """
$ anyscale llm dataset download train.jsonl
0:00:00 100% ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 711.0 kB / 711.0 kB Downloading 'train.jsonl'

Download complete!

{"messages":[{"content":"hi","role":"user"},{"content":"Hi! How can I help?","role":"assistant"}]}
...
{"messages":[{"content":"bye","role":"user"},{"content":"Goodbye!","role":"assistant"}]}
"""

RESOURCE_QUOTAS_CREATE_EXAMPLE = """
$ anyscale resource-quota create -n my-resource-quota --cloud my-cloud --project my-project --user-email someone@myorg.com --num-instances 100 --num-cpus 1000 --num-gpus 50 --num-accelerators A10G 10 --num-accelerators A100-80G 0
(anyscale +2.5s) Name: my-resource-quota
Cloud name: my-cloud
Project name: my-project
User email: someone@myorg.com
Number of CPUs: 1000
Number of instances: 100
Number of GPUs: 50
Number of accelerators: {'A10G': 10, 'A100-80G': 0}
(anyscale +2.5s) Resource quota created successfully ID: rsq_abcdef
"""

RESOURCE_QUOTAS_LIST_EXAMPLE = """
$ anyscale resource-quota list --cloud my-cloud
Resource quotas:
ID       NAME               CLOUD ID    PROJECT ID  USER ID     IS ENABLED    CREATED AT    DELETED AT    QUOTA
rsq_123  resource-quota-1   cld_abcdef  prj_abcdef  usr_abcdef  True          09/11/2024                  {'num_accelerators': {'A100-80G': 0, 'A10G': 10},
                                                                                                           'num_cpus': 1000,
                                                                                                           'num_gpus': 50,
                                                                                                           'num_instances': 100}
rsq_456  resource-quota-2   cld_abcdef              usr_abcdef  True          09/10/2024                  {'num_accelerators': {}, 'num_cpus': None, 'num_gpus': None, 'num_instances': 2}
rsq_789  resource-quota-3   cld_abcdef                          False         09/05/2024                  {'num_accelerators': {'A10G': 1},
                                                                                                            'num_cpus': None,
                                                                                                            'num_gpus': None,
                                                                                                            'num_instances': None}
"""

RESOURCE_QUOTAS_ENABLE_EXAMPLE = """
$ anyscale resource-quota enable --id rsq_abcdef
(anyscale +1.2s) Enabled resource quota with ID rsq_abcdef successfully.
"""

RESOURCE_QUOTAS_DISABLE_EXAMPLE = """
$ anyscale resource-quota disable --id rsq_abcdef
(anyscale +1.4s) Disabled resource quota with ID rsq_abcdef successfully.
"""

RESOURCE_QUOTAS_DELETE_EXAMPLE = """
$ anyscale resource-quota delete --id rsq_abcdef
(anyscale +1.0s) Resource quota with ID rsq_abcdef deleted successfully.
"""

COMPUTE_CONFIG_CREATE_EXAMPLE = """
$ anyscale compute-config create -n my-compute-config -f compute_config.yaml
(anyscale +3.7s) Created compute config: 'my-compute-config:1'
(anyscale +3.7s) View the compute config in the UI: 'https://console.anyscale.com/v2/...'

$cat compute_config.yaml
head_node:
  instance_type: m5.8xlarge
worker_nodes:
- instance_type: m5.8xlarge
  min_nodes: 5
  max_nodes: 5
  market_type: ON_DEMAND # (Optional) Defaults to ON_DEMAND
- instance_type: g4dn.xlarge
  min_nodes: 1
  max_nodes: 10
  market_type: PREFER_SPOT # (Optional) Defaults to ON_DEMAND
min_resources: # (Optional) Defaults to no minimum.
  CPU: 1
  GPU: 1
max_resources: # (Optional) Defaults to no maximum.
  CPU: 6
  GPU: 1
"""

COMPUTE_CONFIG_GET_EXAMPLE = """
$ anyscale compute-config get -n my-compute-config
name: my-compute-config:1
id: cpt_buthu4glxj3azv4e287jad3ya3
config:
  cloud: aviary-prod-us-east-1
  head_node:
    instance_type: m5.8xlarge
    resources:
      CPU: 0
      GPU: 0
  worker_nodes:
  - instance_type: m5.8xlarge
    name: m5.8xlarge
    min_nodes: 5
    max_nodes: 5
    market_type: ON_DEMAND
  - instance_type: g4dn.xlarge
    name: g4dn.xlarge
    min_nodes: 1
    max_nodes: 10
    market_type: PREFER_SPOT
  min_resources:
    CPU: 1
    GPU: 1
  max_resources:
    CPU: 6
    GPU: 1
  enable_cross_zone_scaling: false
  flags: {}
"""

COMPUTE_CONFIG_ARCHIVE_EXAMPLE = """
$ anyscale compute-config archive -n my-compute-config
(anyscale +2.3s) Compute config is successfully archived.
"""

IMAGE_BUILD_EXAMPLE = """
$ anyscale image build -f my.Dockerfile -n my-image --ray-version 2.21.0
(anyscale +2.8s) Building image. View it in the UI: https://console.anyscale.com/v2/...
(anyscale +1m53.0s) Waiting for image build to complete. Elapsed time: 102 seconds.
(anyscale +1m53.0s) Image build succeeded.
Image built successfully with URI: anyscale/image/my-image:1

$ cat my.Dockerfile
FROM anyscale/ray:2.21.0-py39
RUN pip install --no-cache-dir pandas
"""

IMAGE_GET_EXAMPLE = """
$ anyscale image get -n my-image
uri: anyscale/image/my-image:1
status: SUCCEEDED
ray_version: 2.21.0
"""

IMAGE_REGISTER_EXAMPLE = """
$ anyscale image register --image-uri docker.io/myrepo/image:v2 --name mycoolimage --ray-version 2.30.0
Image registered successfully with URI: anyscale/image/mycoolimage:1
"""



import os
from typing import Optional, Tuple

from anyscale.client.openapi_client.api.default_api import DefaultApi as InternalApi
from anyscale.client.openapi_client.models import (
    Cloud,
    ExperimentalWorkspace,
)
from anyscale.sdk.anyscale_client.api.default_api import DefaultApi as ExternalApi
from anyscale.sdk.anyscale_client.models import Project


WORKSPACE_ID_ENV_VAR = "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID"

# TODO (sumanthrh): Currently this same functionality of getting workspace id is repeated in many places, we should try to use this util everywhere
def get_current_workspace_id() -> Optional[str]:
    """Returns the ID of the workspace the client is running in (or `None`)."""
    return os.environ.get(WORKSPACE_ID_ENV_VAR, None)


def source_cloud_id_and_project_id(
    *,
    internal_api: InternalApi,
    external_api: ExternalApi,
    cloud_id: Optional[str],
    project_id: Optional[str],
) -> Tuple[str, str]:
    """
        Returns `Tuple[cloud_id, project_id]` based on optionally provided `cloud_id` and `project_id`.

        - If neither `cloud_id` nor `project_id` are provided, the default cloud and its default project are used
        (unless inside a workspace)
        - If both `cloud_id` and `project_id` are provided, we verify that the project is under the cloud.

        Note that the relationship hierarchy is:
        ```
        Cloud -----------<- Project
        1                     Many
        ```
        - Each cloud will have at least one project (the default project).
        - Each project will belong to exactly one cloud.
        """
    if not cloud_id:
        if project_id:
            # 1. No cloud, project
            # (Use the cloud of the project)
            project: Project = external_api.get_project(project_id).result
            cloud_id = project.parent_cloud_id
        else:
            # 2. No cloud, no project
            # (Use the default cloud and its default project)
            # (unless we are in an Anyscale workspace, then use the workspace's cloud and project)
            workspace_id = get_current_workspace_id()
            is_anyscale_workspace = workspace_id is not None
            if is_anyscale_workspace:
                workspace: ExperimentalWorkspace = internal_api.get_workspace_api_v2_experimental_workspaces_workspace_id_get(
                    workspace_id
                ).result
                cloud_id = workspace.cloud_id
                project_id = workspace.project_id
            else:
                cloud: Cloud = external_api.get_default_cloud().result
                cloud_id = cloud.id
                project = external_api.get_default_project(
                    parent_cloud_id=cloud_id
                ).result
                project_id = project.id
    else:
        if not project_id:
            # 3. Cloud, no project
            # (Use the default project of the cloud)
            project = external_api.get_default_project(parent_cloud_id=cloud_id).result
            project_id = project.id
        else:
            # 4. Cloud, project
            # (Verify that the project is in the cloud)
            project = external_api.get_project(project_id).result
            assert (
                project.parent_cloud_id == cloud_id
            ), f"Project {project_id} is not in cloud {cloud_id}."
    return (cloud_id, project_id)  # type: ignore



from datetime import datetime
import os
import re


def gen_valid_name(prefix: str = "autogenerated") -> str:
    """Generate a name for anyscale resources."""
    INVALID_NAME_REGEX_PATTERN = r"[^A-Za-z0-9_-]"

    generated_name = f"{prefix}-{datetime.now().isoformat()}"
    generated_name = re.sub(INVALID_NAME_REGEX_PATTERN, "-", generated_name)
    return generated_name


def get_full_name():
    """Gets the full name of the user as a lowercase `snake_case` string.
    Uses the Linux username if full name is not available.

    If still not available, returns `"default"`.

    eg. `get_full_name() -> "john_doe"`
    """
    full_name = (
        os.getenv("ANYSCALE_USERNAME")
        or os.getenv("USERNAME")
        or os.getenv("USER")
        or "default"
    )
    # Join the full name with underscores
    full_name = "_".join(full_name.split())
    # Lowercase the full name
    full_name = full_name.lower()
    return full_name



import os
from typing import Optional

import click
from rich import print as rprint

import anyscale
from anyscale.commands import command_examples
from anyscale.commands.util import AnyscaleCommand


@click.group("dataset", help="Dataset files stored on your Anyscale cloud.")
def dataset_cli():
    pass


@dataset_cli.command(
    name="get",
    short_help="Retrieves metadata about a dataset.",
    cls=AnyscaleCommand,
    is_alpha=True,
    example=command_examples.LLM_DATASET_GET_EXAMPLE,
)
@click.argument(
    "name", required=True,
)
@click.option(
    "--version",
    "-v",
    required=False,
    type=int,
    help="Version of the dataset. "
    "If a negative integer is provided, the dataset returned is this many versions back of the latest version. "
    "Default: Latest version.",
)
@click.option(
    "--project",
    required=False,
    help="Name of the Anyscale project that the dataset belongs to. "
    "If not provided, all projects will be searched.",
)
def get_dataset(
    name: str, version: Optional[int], project: Optional[str],
):
    """
    Retrieves metadata about a dataset.

    NAME = Name of the dataset

    Example usage:

        anyscale llm dataset get my_first_dataset

    Retrieve the second latest version of the dataset:

        anyscale llm dataset get my_first_dataset -v -1
    """
    dataset = anyscale.llm.dataset.get(name, version, project)
    rprint(dataset)


@dataset_cli.command(
    name="upload",
    short_help="Upload a dataset to your Anyscale cloud.",
    cls=AnyscaleCommand,
    is_alpha=True,
    example=command_examples.LLM_DATASET_UPLOAD_EXAMPLE,
)
@click.argument("dataset_file", required=True)
@click.option(
    "--name",
    "-n",
    required=True,
    help="Name of a new dataset, or an existing dataset, to upload a new version of.",
)
@click.option(
    "--description", required=False, help="Description of the dataset version.",
)
@click.option(
    "--cloud",
    required=False,
    help="Name of the Anyscale cloud to upload a new dataset to. "
    "If not provided, the default cloud will be used.",
)
@click.option(
    "--project",
    required=False,
    help="Name of the Anyscale project to upload a new dataset to. "
    "If not provided, the default project of the cloud will be used.",
)
def upload_dataset(
    dataset_file: str,
    name: str,
    description: Optional[str],
    cloud: Optional[str],
    project: Optional[str],
):
    """
    Uploads a dataset, or a new version of a dataset, to your Anyscale cloud.

    DATASET_FILE = Path to the dataset file to upload

    Example usage:

        anyscale llm dataset upload path/to/my_dataset.jsonl -n my_first_dataset

        anyscale llm dataset upload my_dataset.jsonl -n second_dataset.jsonl

        anyscale llm dataset upload my_dataset2.jsonl -n second_dataset.jsonl --description 'added 3 lines'

    \b
    NOTE:
    If you are uploading a new version, have run this from within an Anyscale workspace,
    and neither `--cloud` nor `--project` is provided, the cloud and project of the workspace will be used.
    """
    dataset = anyscale.llm.dataset.upload(
        dataset_file, name, description=description, cloud=cloud, project=project,
    )
    rprint(dataset)


@dataset_cli.command(
    name="download",
    short_help="Download a dataset.",
    cls=AnyscaleCommand,
    is_alpha=True,
    example=command_examples.LLM_DATASET_DOWNLOAD_EXAMPLE,
)
@click.argument(
    "name", required=True,
)
@click.option(
    "--version",
    "-v",
    required=False,
    type=int,
    help="Version of the dataset to download. "
    "If a negative integer is provided, the dataset returned is this many versions back of the latest version. "
    "Default: Latest version.",
)
@click.option(
    "--project",
    required=False,
    help="Name of the Anyscale project to download the dataset from. "
    "If not provided, all projects will be searched.",
)
@click.option(
    "--output",
    "-o",
    required=False,
    help="Path to save the downloaded dataset to."
    "If not provided, the dataset contents will be printed to the terminal.",
)
def download_dataset(
    name: str, version: Optional[int], project: Optional[str], output: Optional[str],
):
    """
    Downloads a dataset from your Anyscale cloud.

    NAME = Name of the dataset to download

    Prints the dataset contents to the terminal by default.

    Example usage:

        anyscale llm dataset download my_first_dataset.jsonl

    Save the dataset to a file:

        anyscale llm dataset download my_dataset.jsonl -o ~/Downloads/my_dataset.jsonl

    Retrieve the second latest version of the dataset:

        anyscale llm dataset download my_dataset.jsonl -v -1
    """
    if output:
        output = os.path.expanduser(output)
        if os.path.exists(output):
            raise click.ClickException(f"File already exists at '{output}'")
    downloaded_contents = anyscale.llm.dataset.download(name, version, project)
    if output:
        # Create the parent directory if it doesn't exist
        parent_dir = os.path.dirname(output)
        if parent_dir:
            os.makedirs(parent_dir, exist_ok=True)
        with open(output, "wb") as output_file:
            output_file.write(downloaded_contents)
        rprint(f"Dataset '{name}' downloaded to '{output}'")
        return
    print(downloaded_contents.decode())
